package org.xbib.interlibrary.http.server;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.xbib.net.http.server.application.Application;
import org.xbib.net.http.server.application.BaseApplicationModule;
import org.xbib.net.http.server.route.HttpRouterContext;
import org.xbib.settings.Settings;

public class InterlibraryServerApplicationModule extends BaseApplicationModule {

    private static final Logger logger = Logger.getLogger(InterlibraryServerApplicationModule.class.getName());

    public InterlibraryServerApplicationModule(Application application,
                                               String name,
                                               Settings settings) {
        super(application, name, settings);
        Properties properties = new Properties();
        properties.put("profile", application.getProfile());
        try (InputStream inputStream = getClass().getResourceAsStream("application.properties")) {
            if (inputStream != null) {
                properties.load(inputStream);
                properties.forEach((key, value) -> application.getAttributes().put(key.toString(), value));
            }
        } catch (IOException e) {
            logger.log(Level.WARNING, e.getMessage(), e);
        }
        logger.log(Level.INFO, "application attributes = " + application.getAttributes());
    }

    @Override
    public void onOpen(HttpRouterContext httpRouterContext) {
        super.onOpen(httpRouterContext);
        // make all modules available in context
        application.getModules().forEach(module -> httpRouterContext.getAttributes().put(module.getName(), module));
    }
}
