package org.xbib.interlibrary.http.server;

import org.xbib.net.ParameterException;
import org.xbib.net.Resource;
import org.xbib.net.http.HttpResponseStatus;
import org.xbib.net.http.j2html.J2HtmlResource;
import org.xbib.net.http.j2html.J2HtmlResourceHandler;
import org.xbib.net.http.server.resource.HtmlTemplateResourceHandler;
import org.xbib.net.http.server.route.HttpRouterContext;

import java.io.IOException;

import static org.xbib.j2html.TagCreator.attrs;
import static org.xbib.j2html.TagCreator.body;
import static org.xbib.j2html.TagCreator.div;
import static org.xbib.j2html.TagCreator.document;
import static org.xbib.j2html.TagCreator.h1;
import static org.xbib.j2html.TagCreator.head;
import static org.xbib.j2html.TagCreator.hr;
import static org.xbib.j2html.TagCreator.html;
import static org.xbib.j2html.TagCreator.link;
import static org.xbib.j2html.TagCreator.meta;
import static org.xbib.j2html.TagCreator.p;
import static org.xbib.j2html.TagCreator.title;

public class ServerStatusHandler extends J2HtmlResourceHandler {

    private final HttpResponseStatus responseStatus;

    public ServerStatusHandler(HttpResponseStatus responseStatus) {
        this.responseStatus = responseStatus;
    }

    private HttpResponseStatus getHttpResponseStatus() {
        return responseStatus;
    }

    @Override
    protected Resource createResource(HttpRouterContext httpRouterContext)
            throws IOException, ParameterException {
        return new ServerStatusResource(this, httpRouterContext);
    }

    protected class ServerStatusResource extends J2HtmlResource {

        protected ServerStatusResource(HtmlTemplateResourceHandler templateResourceHandler,
                                       HttpRouterContext httpRouterContext)
                throws IOException, ParameterException {
            super(templateResourceHandler, httpRouterContext);
        }

        @Override
        protected HttpResponseStatus getResponseStatus() {
            return getHttpResponseStatus();
        }

        @Override
        protected String renderHtml(HttpRouterContext context) throws ParameterException {
            BaseView mainView = new BaseView(this, application, request);
            String message = context.getAttributes().get(String.class, "_message");
            if (message == null) {
                message = getHttpResponseStatus().codeAsText() + " " + getHttpResponseStatus().reasonPhrase();
            }
            return document(html(
                    head(title(message),
                            meta().attr("charset", "utf-8"),
                            meta().attr("name", "viewport").attr("content", "width=device-width, initial-scale=1.0"),
                            link().attr("rel", "stylesheet").attr("href", mainView.bootstrapCss()),
                            link().attr("rel", "stylesheet").attr("href", mainView.localCss())),
                    body(
                            div(attrs(".container"),
                                    div(attrs(".p-5 .round-lg .m-3"),
                                            h1(attrs(".display-3"), "HTTP " + getResponseStatus().codeAsText()),
                                            hr(attrs(".my-4")),
                                            p(message)
                                    )
                            )
                    )
            ));
        }
    }
}
